&routine event

&rem **** Generates features given route coverage and event table
&rem **** To keep things simple, error checking is minimal

&if &eq "x%-1" "x" &or &eq "x%-1" "x/?" &do
   &r ev_use
   &return 0
&end

&rem **** arguments

&define in_cover -1 &var
&define event_table -2 &var
&define out_cover -3 &var
&define from_item -4 &var
&define type -5 &var
&define to_item -6 &var

&rem **** other local variables

&define begin -11 &var
&define end -12 &var
&define id -13 &var
&define temp -19 &var

&rem **** global variables

&define pos 11 &var
&define x 12 &var
&define y 13 &var
&define vdist 14 &var
&define vx 15 &var
&define vy 16 &var

&rem **** create temporary files

&if &fn t$in.ung &do
   & DEL t$in.ung
&end
&type "Creating coordinate list..."
UNGEN LINE [in_cover] t$in.ung
&if &fn t$ev.dbf &do
   & DEL t$ev.dbf
&end
&type "Creating position list..."
COPYINFO [event_table] t$ev
&openw t$tab.sml
&write "SEL t$ev"
&write "SORT [from_item]"
&write "&openw t$ev.lis"
&if &eq [type] POINT &do
   &write "LIST [from_item]"
&else
   &write "LIST [from_item], [to_item]"
&end
&write "&closew"
&write "Q"
&closew
TABLES t$tab.sml

&rem **** create GENERATE file

&openw t$out.ung
&open t$ev.lis error

&rem **** skip first line of event list

&read [temp] error
&open t$in.ung error 2
&sv [id] 1
&sv [pos] 0
&sv [x] nil
&sv [y] nil
&sv [vdist] 0
&while &do
   &read [temp] [break] 1
   &type "Processing event [id]..."
   &extract [begin] [temp] 1
   &if &eq [type] LINE &do
      &extract [end] [temp] 2
   &end

   &rem **** reset crawler if begin < pos

   &if &ne [pos] [begin] &and &eq %<[pos] max [begin]> [pos] &do
      &sv [pos] 0
      &sv [x] nil
      &sv [vdist] 0
      &close 2
      &open t$in.ung error 2
   &end

   &rem **** find begin position

   &r ev_crawl [begin] POINT
   &if &eq [type] POINT &do
      &write "[id] [x] [y]"
   &else
      &write "[id]"
      &write "[x] [y]"
   &end
   &if &eq [type] LINE &do

      &rem **** find end position

      &r ev_crawl [end] LINE
      &write "[x] [y]"
      &write "END"
   &end
   &inc [id]
&end
&write "END"
&close &all
&closew

&rem **** generate coverage

&openw t$gen.sml
&write "INPUT t$out.ung"
&if &eq [type] POINT &do
   &write "POINTS"
   &sv [temp] PAT
&else
   &write "LINES"
   &sv [temp] AAT
&end
&write "Q"
&closew
GENERATE [out_cover] t$gen.sml
BUILD [out_cover] [type]
JOINITEM [out_cover].[temp] t$ev [out_cover].[temp] [out_cover]_ID [out_cover]_ID LINK

&rem **** clean up and exit

& DEL t$*.*
&type "Done."
&return 1

&rem **** I/O error trap

&label error
&type "I/O error: EVENT"
&return 0

&rem ******************************************************************

&routine ev_crawl

&rem **** arguments

&define goal -1 &var
&define type -2 &var

&rem **** local variables

&define dx -15 &var
&define dy -16 &var
&define d1 -17 &var
&define d2 -18 &var
&define temp -19 &var

&rem **** global variables

&define pos 11 &var
&define x 12 &var
&define y 13 &var
&define vdist 14 &var
&define vx 15 &var
&define vy 16 &var

&rem **** if goal is 0, read first coordinate and return it

&if &eq [goal] 0 &do
   &sv [temp] x
   &while &nc "[temp]" "." &do
      &read [temp] [break] 2
   &end
   &extract [x] [temp] 1
   &extract [y] [temp] 2
   &value [vx] [x]
   &value [vy] [y]
   &return 1
&end

&rem **** if x is nil, read x and y

&if &eq "x[x]" "xnil" &do
   &sv [temp] x
   &while &nc "[temp]" "." &do
      &read [temp] [break] 2
   &end
   &extract [x] [temp] 1
   &extract [y] [temp] 2
&else

   &rem **** if goal <= vdist then calculate x and y and return

   &if &eq %<[goal] min [vdist]> [goal] &do
      &cv [d1] [vdist] - [pos]
      &cv [d2] [goal] - [pos]
      &cv [dx] [vx] - [x]
      &cv [dy] [vy] - [y]
      &cv [x] [x] + ( [d2] / [d1] * [dx] )
      &cv [y] [y] + ( [d2] / [d1] * [dy] )
      &cv [pos] [pos] + [d2]
      &return 1
   &end

   &rem **** move crawler to vdist

   &value [pos] [vdist]
   &value [x] [vx]
   &value [y] [vy]
   &if &eq [type] LINE &do
       &write "[x] [y]"
   &end
&end

&rem **** crawl along coordinate list until goal is met

&while &eq %<[vdist] min [goal]> [vdist] &do
   &sv [temp] x
   &while &nc "[temp]" "." &do
      &read [temp] [break] 2
   &end
   &if &eq "x[temp]" "xEND" &do

      &rem **** goal is beyond end of route

      &type "WARNING: position [goal] truncated to [vdist]"
      &sv [x] [vx]
      &sv [y] [vy]
      &sv [pos] [vdist]
      &return 1
   &end
   &extract [vx] [temp] 1
   &extract [vy] [temp] 2
   &cv [dx] [vx] - [x]
   &cv [dy] [vy] - [y]
   &cv [d2] ( ( [dx] * [dx] ) + ( [dy] * [dy] ) ) ** 0.5
   &cv [vdist] [pos] + [d2]
   &if &eq %<[goal] min [vdist]> [goal] &do

       &rem **** calculate x and y and return

       &cv [d1] [vdist] - [pos]
       &cv [d2] [goal] - [pos]
       &cv [dx] [vx] - [x]
       &cv [dy] [vy] - [y]
       &cv [x] [x] + ( [d2] / [d1] * [dx] )
       &cv [y] [y] + ( [d2] / [d1] * [dy] )
       &cv [pos] [pos] + [d2]
       &return 1
   &end
   &value [pos] [vdist]
   &value [x] [vx]
   &value [y] [vy]
   &if &eq [type] LINE &do
       &write "[x] [y]"
   &end
&end
&return 1

&rem **** I/O error trap

&label error
&type "I/O error: EV_CRAWL"
&return 0

&rem ******************************************************************

&routine ev_use

&rem **** display usage

&label usage

&delim < >
&type " "
&type "Usage: &r EVENT [in_cover] [event_table] [out_cover] [from_item]"
&type "                {POINT/LINE} {to_item}"
&delim [ ]
&return 0
